CREATE OR REPLACE PACKAGE cust_auth_sec
AS
   FUNCTION custom_auth (p_username IN VARCHAR2, p_password IN VARCHAR2)
      RETURN BOOLEAN;

   FUNCTION custom_hash (p_username IN VARCHAR2, p_password IN VARCHAR2)
      RETURN VARCHAR2;

   FUNCTION is_group_member (p_usr_id NUMBER, p_grp_id IN NUMBER)
      RETURN BOOLEAN;

   FUNCTION is_group_member (p_usr_name VARCHAR2, p_grp_name IN VARCHAR2)
      RETURN BOOLEAN;
END;
/


CREATE OR REPLACE PACKAGE BODY cust_auth_sec
AS
   FUNCTION custom_hash (p_username IN VARCHAR2, p_password IN VARCHAR2)
      RETURN VARCHAR2
   IS
      l_password   VARCHAR2 (4000);
      l_salt       VARCHAR2 (4000) := 'ZOKGBC6F4KFVJMM3NDWRZSTGXZ4CUX';
   BEGIN
      l_password :=
         UTL_RAW.cast_to_raw
             (DBMS_OBFUSCATION_TOOLKIT.md5 (input_string      =>    p_password
                                                                 || SUBSTR
                                                                       (l_salt,
                                                                        10,
                                                                        13
                                                                       )
                                                                 || p_username
                                                                 || SUBSTR
                                                                       (l_salt,
                                                                        4,
                                                                        10
                                                                       )
                                           )
             );
      RETURN l_password;
   END;

   FUNCTION custom_auth (p_username IN VARCHAR2, p_password IN VARCHAR2)
      RETURN BOOLEAN
   IS
      l_password          VARCHAR2 (4000);
      l_stored_password   VARCHAR2 (4000);
      l_expires_on        DATE;
      l_count             NUMBER;
   BEGIN
      SELECT COUNT (*)
        INTO l_count
        FROM cust_auth_users
       WHERE usr_name = p_username;

      IF l_count > 0
      THEN
         SELECT usr_password_hash
           INTO l_stored_password
           FROM cust_auth_users
          WHERE usr_name = p_username;

         l_password := custom_hash (p_username, p_password);

         IF l_password = l_stored_password
         THEN
            RETURN TRUE;
         ELSE
            RETURN FALSE;
         END IF;
      ELSE
         RETURN FALSE;
      END IF;
   END;

   FUNCTION get_usr_id (p_usr_name IN VARCHAR2)
      RETURN NUMBER
   IS
      l_usr_id   cust_auth_users.usr_id%TYPE;
   BEGIN
      SELECT usr_id
        INTO l_usr_id
        FROM cust_auth_users
       WHERE usr_name = p_usr_name;

      RETURN l_usr_id;
   END;

   FUNCTION get_grp_id (p_grp_name IN VARCHAR2)
      RETURN NUMBER
   IS
      l_grp_id   cust_auth_groups.grp_id%TYPE;
   BEGIN
      SELECT grp_id
        INTO l_grp_id
        FROM cust_auth_groups
       WHERE grp_name = p_grp_name;

      RETURN l_grp_id;
   END;

   FUNCTION is_group_member (p_usr_id NUMBER, p_grp_id IN NUMBER)
      RETURN BOOLEAN
   IS
      l_usr_id   NUMBER;
   BEGIN
      SELECT usr_id
        INTO l_usr_id
        FROM cust_auth_users_groups
       WHERE usr_id = p_usr_id AND grp_id = p_grp_id;

      RETURN TRUE;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END;

   FUNCTION is_group_member (p_usr_name VARCHAR2, p_grp_name IN VARCHAR2)
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN is_group_member
                            (p_usr_id      => get_usr_id
                                                     (p_usr_name      => p_usr_name),
                             p_grp_id      => get_grp_id
                                                     (p_grp_name      => p_grp_name)
                            );
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END;
END;
/